/*
 * Copyright (C) 2009 Splitted-Desktop Systems. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "config.h"
#include "va.h"
#include "va_backend.h"
#include "va_x11.h"
#include "va_glx.h"

typedef struct VADisplayContextGLX *VADisplayContextGLXP;

struct VADisplayContextGLX {
    void (*vaDestroy)(VADisplayContextP ctx);
};

static void va_DisplayContextDestroy(VADisplayContextP pDisplayContext)
{
    VADisplayContextGLXP pDisplayContextGLX;

    if (!pDisplayContext)
        return;

    pDisplayContextGLX = pDisplayContext->opaque;
    if (pDisplayContextGLX) {
        if (pDisplayContextGLX->vaDestroy)
            pDisplayContextGLX->vaDestroy(pDisplayContext);
        free(pDisplayContextGLX);
        pDisplayContext->opaque = NULL;
    }
}

VADisplay vaGetDisplayGLX(Display *native_dpy)
{
    VADisplay            dpy                = NULL;
    VADisplayContextP    pDisplayContext    = NULL;
    VADisplayContextGLXP pDisplayContextGLX = NULL;
    VADriverContextP     pDriverContext;

    dpy = vaGetDisplay(native_dpy);
    if (!dpy)
        return NULL;
    pDisplayContext = (VADisplayContextP)dpy;
    pDriverContext  = pDisplayContext->pDriverContext;

    pDisplayContextGLX = calloc(1, sizeof(*pDisplayContextGLX));
    if (!pDisplayContextGLX)
        goto error;

    pDisplayContextGLX->vaDestroy = pDisplayContext->vaDestroy;
    pDisplayContext->vaDestroy    = va_DisplayContextDestroy;
    pDisplayContext->opaque       = pDisplayContextGLX;
    return dpy;

error:
    free(pDisplayContextGLX);
    pDisplayContext->vaDestroy(pDisplayContext);
    return NULL;
}

VAStatus vaCopySurfaceToTextureGLX(
    VADisplay    dpy,
    VASurfaceID  surface,
    GLuint       texture,
    unsigned int flags
)
{
    VADisplayContextP pDisplayContext = (VADisplayContextP)dpy;
    VADriverContextP ctx;

    if (!pDisplayContext)
        return VA_STATUS_ERROR_INVALID_DISPLAY;
    if (!pDisplayContext->vaIsValid)
        return VA_STATUS_ERROR_INVALID_DISPLAY;
    if (!pDisplayContext->vaIsValid(pDisplayContext))
        return VA_STATUS_ERROR_INVALID_DISPLAY;

    ctx = pDisplayContext->pDriverContext;
    if (!ctx)
        return VA_STATUS_ERROR_OPERATION_FAILED;

    if (ctx->vtable.vaCopySurfaceToTextureGLX)
        return ctx->vtable.vaCopySurfaceToTextureGLX(ctx, surface, texture, flags);

    /* XXX: implement with the GLX texture-from-pixmap + FBO extension? */
    return VA_STATUS_ERROR_UNIMPLEMENTED;
}

VAStatus vaBindSurfaceToTextureGLX(
    VADisplay    dpy,
    VASurfaceID  surface,
    unsigned int texture,
    unsigned int flags
)
{
    VADisplayContextP pDisplayContext = (VADisplayContextP)dpy;
    VADriverContextP ctx;

    if (!pDisplayContext)
        return VA_STATUS_ERROR_INVALID_DISPLAY;
    if (!pDisplayContext->vaIsValid)
        return VA_STATUS_ERROR_INVALID_DISPLAY;
    if (!pDisplayContext->vaIsValid(pDisplayContext))
        return VA_STATUS_ERROR_INVALID_DISPLAY;

    ctx = pDisplayContext->pDriverContext;
    if (!ctx)
        return VA_STATUS_ERROR_OPERATION_FAILED;

    if (ctx->vtable.vaBindSurfaceToTextureGLX) {
        if (!ctx->vtable.vaReleaseSurfaceFromTextureGLX)
            return VA_STATUS_ERROR_UNIMPLEMENTED;
        return ctx->vtable.vaBindSurfaceToTextureGLX(ctx, surface, texture, flags);
    }

    /* XXX: implement with the GLX texture-from-pixmap? */
    return VA_STATUS_ERROR_UNIMPLEMENTED;
}

VAStatus vaReleaseSurfaceFromTextureGLX(
    VADisplay    dpy,
    VASurfaceID  surface
)
{
    VADisplayContextP pDisplayContext = (VADisplayContextP)dpy;
    VADriverContextP ctx;

    if (!pDisplayContext)
        return VA_STATUS_ERROR_INVALID_DISPLAY;
    if (!pDisplayContext->vaIsValid)
        return VA_STATUS_ERROR_INVALID_DISPLAY;
    if (!pDisplayContext->vaIsValid(pDisplayContext))
        return VA_STATUS_ERROR_INVALID_DISPLAY;

    ctx = pDisplayContext->pDriverContext;
    if (!ctx)
        return VA_STATUS_ERROR_OPERATION_FAILED;

    if (ctx->vtable.vaReleaseSurfaceFromTextureGLX) {
        if (!ctx->vtable.vaBindSurfaceToTextureGLX)
            return VA_STATUS_ERROR_UNIMPLEMENTED;
        return ctx->vtable.vaReleaseSurfaceFromTextureGLX(ctx, surface);
    }

    /* XXX: implement with the GLX texture-from-pixmap? */
    return VA_STATUS_ERROR_UNIMPLEMENTED;
}
